from notion_client.helpers import get_id
from notion2md.exporter import *
import os
import argparse

def cli():
    parser = argparse.ArgumentParser(description="Notion2md: Notion to Markdown Exporter")
    parser.add_argument('--type','-t',type=str,help="a type of target page: block, page, database",default="block")
    parser.add_argument('--url','-u',type=str,help="an url of target page")
    parser.add_argument('--id','-i',type=str,help="an id of target page")
    parser.add_argument('--path','-p',type=str,help="a relative path of output file")
    parser.add_argument('--name','-n',type=str,help="a custom name of output file")

    args = parser.parse_args()

    def get_url():
        return input("Enter Notion Url: ")

    custom_name = args.name if args.name else ""

    target_id = get_id(args.url) if args.url \
        else ( args.id if args.id \
        else get_id(get_url()))

    output_path = os.path.abspath(args.path) if args.path \
        else os.path.join(os.getcwd(),'notion2md-output')

    target_type_map ={
        'block': block_exporter,
        # 'page': page_exporter,
        # 'database': database
    }

    target_type_map[args.type](target_id,output_path,custom_name)

    # page_exporter()

    # database_exporter()


from notion2md.client_handler import notion_client_object
from notion2md.convertor.block import blocks_convertor
import os
import time

os.system("")

class style():
    BLACK = '\033[30m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    MAGENTA = '\033[35m'
    CYAN = '\033[36m'
    WHITE = '\033[37m'
    UNDERLINE = '\033[4m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    RESET = '\033[0m'

def fprint(left,right):
    print(f"{style.GREEN}{style.BOLD}{left+' ':>12}{style.RESET}{right}")

def block_exporter(target_id:str,output_path="notion2md-output",custom_name=""):
    start_time = time.time()
    #Directory Checking and Creating
    if not os.path.exists(output_path):
        os.mkdir(output_path)
    #Get title from parent page block
    block_title = custom_name if custom_name \
        else notion_client_object.blocks.retrieve(target_id)['child_page']['title']
    #Get actual blocks
    print()
    fprint("Retrieving",f"blocks from '{target_id}'")
    blocks = notion_client_object.blocks.children.list(target_id)['results']
    #Write(Export) Markdown file
    with open(os.path.join(output_path,block_title+'.md'),'w',encoding="utf-8") as output:
        output.write(blocks_convertor(blocks))
    #Result and Time Check
    fprint("Converted", f"{str(len(blocks))} blocks to markdown in {time.time() - start_time:.2f}s")
    fprint("Exported", f'"{block_title}.md" in "{os.path.abspath(output_path)}/"')
    print()

# page_exporter()

# database_exporter()



from .richtext import richtext_convertor
from notion2md.client_handler import notion_client_object
import concurrent.futures

def paragraph(information:dict) -> str:
    return information['text']

def heading_1(information:dict) -> str:
    return f"# {information['text']}"

def heading_2(information:dict) -> str:
    return f"## {information['text']}"

def heading_3(information:dict) -> str:
    return f"### {information['text']}"

def callout(information:dict) -> str:
    return f"{information['icon']} {information['text']}"

def quote(information:dict) -> str:
    return f"> {information['text']}"

#toggle item will be changed as bulleted list item
def bulleted_list_item(information:dict) -> str:
    return f"- {information['text']}"

# numbering is not supported
def numbered_list_item(information:dict) -> str:
    """
    input: item:dict = {"number":int, "text":str}
    """
    return f"1. {information['text']}"

def to_do(information:dict) -> str:
    """
    input: item:dict = {"checked":bool, "test":str}
    """
    return f"- {'[x]' if information['checked'] else '[ ]'} {information['text']}"

#not yet supported
#child_database will be changed as child page
# def child_page(information:dict) -> str:
#     """
#     input: item:dict = {"id":str,"text":str}
#     """
#     #make_page(information['id'])
#     text = information['text']
#     return f'[{text}]({text})'

def code(information:dict) -> str:
    """
    input: item:dict = {"language":str,"text":str}
    """
    return f"```{information['language']}\n\t{information['text']}\n```"

def embed(information:dict) -> str:
    """
    input: item:dict ={"url":str,"text":str}
    """
    return f"[{information['url']}]({information['url']})"

def image(information:dict) -> str:
    """
    input: item:dict ={"url":str,"text":str,"caption":str}
    """
    #if internal: make_image(url)
    return f"![{information['url']}]({information['url']})\n\n{information['caption']}"

def bookmark(information:dict) -> str:
    """
    input: item:dict ={"url":str,"text":str,"caption":str}
    """
    #if internal: make_image(url)
    return f"[{information['url']}]({information['url']})\n\n{information['caption']}"

def equation(information:dict) -> str:
    return f"$$ {information['text']} $$"

def divider(information:dict) -> str:
    return f"---"

def blank() -> str:
    return "<br/>"

block_type_map = {
    "paragraph": paragraph,
    "heading_1": heading_1,
    "heading_2": heading_2,
    "heading_3": heading_3,
    "callout": callout,
    "toggle":bulleted_list_item,
    "quote": quote,
    "bulleted_list_item": bulleted_list_item,
    "numbered_list_item": numbered_list_item,
    "to_do": to_do,
    # "child_page": child_page,
    "code": code,
    "embed": embed,
    "imgae": image,
    "bookmark": bookmark,
    "equation": equation,
    "divider": divider,
}

def blocks_convertor(blocks:object) -> str:
    outcome_blocks:str = ""
    with concurrent.futures.ThreadPoolExecutor() as executor:
        results = executor.map(block_convertor,blocks)
        outcome_blocks = "".join([result for result in results])
    return outcome_blocks

def information_collector(payload:dict) -> dict:
    information = dict()
    if "text" in payload:
        information['text'] = richtext_convertor(payload['text'])
    if "icon" in payload:
        information['icon'] = payload['icon']['emoji']
    if "checked" in payload:
        information['checked'] = payload['checked']
    if "expression" in payload:
        information['text'] = payload['expression']
    if "url" in payload:
        information['url'] = payload['url']
    if "caption" in payload:
        information['caption'] = richtext_convertor(payload['caption'])
    if "external" in payload:
        information['url'] = payload['external']['url']
    if "language" in payload:
        information['language'] = payload['language']
    return information

def block_convertor(block:object,depth=0) -> str:
    outcome_block:str = ""
    block_type = block['type']
    #Special Case: Block is blank
    if block_type == "paragraph" and not block['has_children'] and not block[block_type]['text']:
        outcome_block = blank() +"\n\n"
    else:
        if block_type in block_type_map:
            outcome_block = block_type_map[block_type](information_collector(block[block_type])) + "\n\n"
        else:
            outcome_block = f"[{block_type} is not supported]\n\n"
        if block['has_children']:
            if block_type == "child_page":
                #call make_child_function
                pass
            else:
                depth += 1
                child_blocks = notion_client_object.blocks.children.list(block_id=block['id'])
                for block in child_blocks['results']:
                    outcome_block += "&nbsp;&nbsp;&nbsp;&nbsp;"*depth + block_convertor(block,depth)
    return outcome_block


from notion_client import Client #,AsyncClient
import os

try:
    notion_client_object = Client(auth=os.environ["NOTION_TOKEN"])
except:
    """
        Token Error

        Welcome to notion2md!

        To get started, you need to save your Notion Integration Token.
        Find your token at

            https://www.notion.so/my-integrations

        Then run shell command:

            $export NOTION_TOKEN="<Your Token>"
        
        If you want to save this environment variable after reboot,
        put upper command in your shell resource(ex: .bashrc or .zshrc)
    """

# notion_async_client = AsyncClient(auth=os.environ["NOTION_TOKEN"])


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notion2md', 'notion2md.convertor']

package_data = \
{'': ['*']}

install_requires = \
['notion-client>=0.7.1']

entry_points = \
{'console_scripts': ['notion2md = notion2md.exporter_client:cli']}

setup_kwargs = {
    'name': 'notion2md',
    'version': '2.2.2',
    'description': 'Notion Markdown Exporter with Python Cli',
    'long_description': '# Notion Markdown Exporter 2.2 [Updated]\n\n[![PyPI version](https://badge.fury.io/py/notion2md.svg)](https://badge.fury.io/py/notion2md)\n\n- Notion Markdown Exporter using **official notion api** by [notion-sdk-py](https://github.com/ramnes/notion-sdk-py)\n\n## API Key(Token)\n\n- Before getting started, create [an integration and find the token](https://www.notion.so/my-integrations). → [Learn more about authorization](https://developers.notion.com/docs/authorization).\n\n- Then save your api key(token) as your os environment variable\n\n```Bash\n$ export NOTION_TOKEN="{your integration token key}"\n```\n\n## Install\n\n```Bash\n$ pip install notion2md\n```\n\n## Useage: Shell Command\n\n![notion2md-options](notion2md-options.png)\n\n```Bash\nnotion2md -p ~/MyBlog/content/posts -u https://notion.so/...\n```\n\n## Usage: Python\n```Python\nfrom notion2md.exporter import block_exporter\n\n#output_path is optional\nblock_exporter("id of notion page","OutPut Path(Relative)")\n```\n\n## To-do\n\n- [ ] Page Exporter\n- [ ] Database Exporter\n- [ ] export file object(image and files)\n- [ ] export child page\n \n## Contribution\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\nPlease make sure to update tests as appropriate.\n\n## License\n[MIT](https://choosealicense.com/licenses/mit/)\n',
    'author': 'echo724',
    'author_email': 'eunchan1001@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/echo724/notion2md.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)



#Link
def link(item:dict):
    """
    input: item:dict ={"content":str,"link":str}
    """
    return f"[{item['content']}]({item['link']['url']})"

#Annotations
def bold(content:str):
    return f"**{content}**"

def italic(content:str):
    return f"*{content}*"

def strikethrough(content:str):
    return f"~~{content}~~"

def underline(content:str):
    return f"<u>{content}</u>"

def code(content:str):
    return f"`{content}`"

def color(content:str,color):
    return f"<span style='color:{color}'>{content}</span>"

def equation(content:str):
    return f"$ {content} $"

annotation_map = {
    "bold": bold,
    "italic": italic,
    "strikethrough": strikethrough,
    "underline": underline,
    "code": code,
}

def richtext_convertor(richtext_list:list) -> str:
    outcome_sentence = ""
    for richtext in richtext_list:
        outcome_word = ""
        plain_text = richtext["plain_text"]
        if richtext['type'] == "equation":
            outcome_word = equation(plain_text)
        else:
            annot = richtext["annotations"]
            if richtext["href"]:
                outcome_word = link(richtext["text"])
            else:
                outcome_word = plain_text
            for key,transfer in annotation_map.items():
                if annot[key]:
                    outcome_word = transfer(outcome_word)
            if annot["color"] != "default":
                outcome_word = color(outcome_word,annot["color"])
        outcome_sentence += outcome_word
    return outcome_sentence